#define MessagePack_Buffer_initialize CBOR_Buffer_initialize
#define MessagePack_Buffer_module_init CBOR_Buffer_module_init
#define MessagePack_Buffer_wrap CBOR_Buffer_wrap
#define MessagePack_Packer_module_init CBOR_Packer_module_init
#define MessagePack_Unpacker_module_init CBOR_Unpacker_module_init
#define MessagePack_core_ext_module_init CBOR_core_ext_module_init
#define MessagePack_pack CBOR_pack
#define MessagePack_unpack CBOR_unpack
#define _msgpack_buffer_append_long_string _CBOR_buffer_append_long_string
#define _msgpack_buffer_expand _CBOR_buffer_expand
#define _msgpack_buffer_feed_from_io _CBOR_buffer_feed_from_io
#define _msgpack_buffer_read_all2 _CBOR_buffer_read_all2
#define _msgpack_buffer_read_from_io_to_string _CBOR_buffer_read_from_io_to_string
#define _msgpack_buffer_shift_chunk _CBOR_buffer_shift_chunk
#define _msgpack_buffer_skip_from_io _CBOR_buffer_skip_from_io
#define _msgpack_rmem_alloc2 _CBOR_rmem_alloc2
#define _msgpack_rmem_chunk_free _CBOR_rmem_chunk_free
#define cMessagePack_Buffer cCBOR_Buffer
#define cMessagePack_Packer cCBOR_Packer
#define cMessagePack_Unpacker cCBOR_Unpacker
#define msgpack_buffer_all_as_string CBOR_buffer_all_as_string
#define msgpack_buffer_all_as_string_array CBOR_buffer_all_as_string_array
#define msgpack_buffer_all_readable_size CBOR_buffer_all_readable_size
#define msgpack_buffer_clear CBOR_buffer_clear
#define msgpack_buffer_destroy CBOR_buffer_destroy
#define msgpack_buffer_flush_to_io CBOR_buffer_flush_to_io
#define msgpack_buffer_init CBOR_buffer_init
#define msgpack_buffer_mark CBOR_buffer_mark
#define msgpack_buffer_read_nonblock CBOR_buffer_read_nonblock
#define msgpack_buffer_read_to_string_nonblock CBOR_buffer_read_to_string_nonblock
#define msgpack_buffer_static_destroy CBOR_buffer_static_destroy
#define msgpack_buffer_static_init CBOR_buffer_static_init
#define msgpack_packer_destroy CBOR_packer_destroy
#define msgpack_packer_init CBOR_packer_init
#define msgpack_packer_mark CBOR_packer_mark
#define msgpack_packer_reset CBOR_packer_reset
#define msgpack_packer_static_destroy CBOR_packer_static_destroy
#define msgpack_packer_static_init CBOR_packer_static_init
#define msgpack_packer_write_array_value CBOR_packer_write_array_value
#define msgpack_packer_write_hash_value CBOR_packer_write_hash_value
#define msgpack_packer_write_value CBOR_packer_write_value
#define msgpack_rmem_destroy CBOR_rmem_destroy
#define msgpack_rmem_init CBOR_rmem_init
#define msgpack_unpacker_destroy CBOR_unpacker_destroy
#define msgpack_unpacker_init CBOR_unpacker_init
#define msgpack_unpacker_mark CBOR_unpacker_mark
#define msgpack_unpacker_peek_next_object_type CBOR_unpacker_peek_next_object_type
#define msgpack_unpacker_read CBOR_unpacker_read
#define msgpack_unpacker_read_array_header CBOR_unpacker_read_array_header
#define msgpack_unpacker_read_container_header CBOR_unpacker_read_container_header
#define msgpack_unpacker_read_map_header CBOR_unpacker_read_map_header
#define msgpack_unpacker_reset CBOR_unpacker_reset
#define msgpack_unpacker_skip CBOR_unpacker_skip
#define msgpack_unpacker_skip_nil CBOR_unpacker_skip_nil
#define msgpack_unpacker_static_destroy CBOR_unpacker_static_destroy
#define msgpack_unpacker_static_init CBOR_unpacker_static_init
